﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Core;

namespace HeuristicLab.Encodings.ParameterConfigurationEncoding.Views {
  public partial class ObjectSelectorDialog<T> : Form where T : IItem {
    private bool initialized;
    private List<IItem> items;
    private IEnumerable<IGrouping<string, T>> selectableItems;

    private T item;
    public T Item {
      get { return item; }
    }

    public ObjectSelectorDialog(IEnumerable<IGrouping<string, T>> selectableItems) {
      this.initialized = false;
      this.items = new List<IItem>();
      this.item = default(T);
      this.selectableItems = selectableItems;
      InitializeComponent();
    }

    #region Events
    private void ObjectSelectorDialog_Load(object sender, EventArgs e) {
      if (!initialized) {
        itemsListView.SmallImageList = new ImageList();

        foreach (IGrouping<string, T> item in selectableItems) {
          ListViewGroup group = new ListViewGroup(item.Key);
          itemsListView.Groups.Add(group);
          foreach (T i in item) {
            items.Add(i);
            string[] columns;
            if (i is INamedItem) {
              INamedItem ni = i as INamedItem;
              columns = new string[] { ni.Name, ni.Description, ni.GetType().Name };
            } else {
              columns = new string[] { i.ItemName, i.ItemDescription, i.GetType().Name };
            }
            ListViewItem value = new ListViewItem(columns, group);
            itemsListView.SmallImageList.Images.Add(i.ItemImage);
            value.ImageIndex = itemsListView.SmallImageList.Images.Count - 1;
            value.Tag = i;
            itemsListView.Items.Add(value);
          }
        }
        for (int i = 0; i < itemsListView.Columns.Count; i++)
          itemsListView.Columns[i].AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent);
        initialized = true;
      }
    }

    private void ObjectSelectorDialog_Shown(object sender, EventArgs e) {
      item = default(T);
    }

    private void okButton_Click(object sender, EventArgs e) {
      if (itemsListView.SelectedItems.Count == 1) {
        item = (T)itemsListView.SelectedItems[0].Tag;
        DialogResult = DialogResult.OK;
        Close();
      }
    }

    private void itemsListView_DoubleClick(object sender, EventArgs e) {
      if (itemsListView.SelectedItems.Count == 1) {
        item = (T)itemsListView.SelectedItems[0].Tag;
        DialogResult = DialogResult.OK;
        Close();
      }
    }

    private void itemsListView_SelectedIndexChanged(object sender, EventArgs e) {
      okButton.Enabled = itemsListView.SelectedItems.Count == 1;
    }
    #endregion
  }
}
